﻿using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation;
using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using System.Data;
using System.Text;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Foundation.Data.Sbo;
using Produmex.Sbo.Logex.Data.Reports.Definitions;

namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {

        /// <summary>
        /// Definitions
        /// </summary>
        public static class Definitions
        {
            /// <summary>
            /// Name of the Impermo report
            /// </summary>
            public const string REPORT_NAME = "ImpermoTegoedBon";

            /// <summary>
            /// number of copies
            /// </summary>
            public const int NUMBER_OF_COPIES = 1;

        }

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            string query = null;
            DataSet dsInvoices = null;
            string msgKey = null;
            ProdumexError pmxError = null;
            int baseDocKey = 0;
            DataSet dsReports = null;
            Collection<ReportParameter> reportParameterCollection = new Collection<ReportParameter>();
            string parameterName = ReportTypeSAPCrystalReportsDefinition.Parameters.DocEntry.NAME;

            string deviceID = session.DeviceInstanceId;
            DataSet dsLocation = null;
            string locationCode = null;
 
            
            foreach (DocumentRef docRef in DocumentRefs.Get())
            {
                if (docRef.DocType == ((int)SboObjectType.SalesDeliveryNote).ToString())
                {
                    try
                    {
                        query = BuildQuery.GetInvoiceOrSalesOrderForSalesDelivery(docRef.DocEntry);
                        dsInvoices = sboProviderService.RunView(false, null, null, query);

                        if (dsInvoices.Tables[0].Rows.Count > 0)
                        {
                            // open records found, launch print action

                            foreach (DataRow  row in dsInvoices.Tables[0].Rows )
                            {
                                baseDocKey = int.Parse(row[PmxSalesInvoiceTable.Columns.DocEntry.NAME].ToString());
                            }

                            // get location dock from where pallets are shipped (for printer)

                            query = BuildQuery.GetLocationCodeForDelivery(docRef.DocEntry);
                            dsLocation = sboProviderService.RunView(false, null, null, query);
                            locationCode = dsLocation.Tables[0].Rows[0][PmxSalesDeliveryNoteLineTable.Columns.StorageLocationCode.NAME].ToString();

                            // look for report 

                            query = BuildQuery.GetReportWithReportName(Definitions.REPORT_NAME);
                            dsReports = sboProviderService.RunView(false, null, null, query);

                            string reportName = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.Name.NAME].ToString();
                            string reportKey = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.Path.NAME].ToString();
                            string reportPageSize = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.PageSizeCode.NAME].ToString();
                            string orentation = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.Orientation.NAME].ToString();
                            string reportTypeCode = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.ReportTypeCode.NAME].ToString();
                            string reportFormat = dsReports.Tables[0].Rows[0][PmxReportTable.Columns.ReportFormat.NAME].ToString();

                            //trigger print
                            ReportParameter param = new ReportParameter(parameterName,baseDocKey);

                            reportParameterCollection.Add(param);

                            sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null,
                                delegate(PmxDbConnection conn, object[] parameters2)
                                {
                                    PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);

                                    return printProv.PrintReport(reportParameterCollection, reportKey, Definitions.NUMBER_OF_COPIES, deviceID, locationCode);

                                });
                        }

                    }



                    catch (ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        msgKey = ex.Message;
                        pmxError = ex.Error;
                    }
                    catch (Exception ex)
                    {
                        s_log.Error("", ex);
                        msgKey = ex.Message;
                    }

                    if (msgKey != null)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", null,
                                "MessageKey", msgKey,
                                "PmxError", pmxError,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                }

            }

            
            return;

        }

        #endregion


        #region BUILD QUERY

    /// <summary>
    /// Static class grouping our native queries
    /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets the invoice or sales order for sales delivery.
            /// </summary>
            /// <param name="docEntry">The doc entry.</param>
            /// <returns></returns>
            public static string GetInvoiceOrSalesOrderForSalesDelivery(int docEntry)
            {
                CultureInfo dbCultureInfo = CultureInfo.InvariantCulture;

                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ISNULL(");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocEntry.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocEntry.NAME));
                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxSalesInvoiceTable.Columns.DocEntry.NAME));

                //strBuilder.Append(" , ISNULL(");
                //strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocStatus.NAME));
                //strBuilder.Append(" , ");
                //strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocStatus.NAME));
                //strBuilder.Append(" ) AS ");
                //strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxSalesInvoiceTable.Columns.DocStatus.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteLineTable.NAME));

                strBuilder.Append(" LEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesInvoiceTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.ObjType.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.BaseType.NAME));
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocEntry.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.BaseEntry.NAME));

                strBuilder.Append(" LEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesOrderTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.ObjType.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.BaseType.NAME));
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocEntry.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.BaseEntry.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.DocEntry.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, dbCultureInfo));

                strBuilder.Append(" AND ISNULL(");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesInvoiceTable.NAME, PmxSalesInvoiceTable.Columns.DocStatus.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocStatus.NAME));
                strBuilder.Append(" ) <> ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxDocumentLineStatusColumnValues.Closed.VALUE, dbCultureInfo));

                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the name of the report with report.
            /// </summary>
            /// <param name="reportName">Name of the report.</param>
            /// <returns></returns>
            public static string GetReportWithReportName(string reportName)
            {
                CultureInfo dbCultureInfo = CultureInfo.InvariantCulture;
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.Name.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.Path.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.PageSizeCode.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.Orientation.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.ReportTypeCode.NAME));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.ReportFormat.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReportTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReportTable.Columns.Name.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(reportName, dbCultureInfo));

                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the location code for delivery.
            /// </summary>
            /// <param name="docEntry">The doc entry.</param>
            /// <returns></returns>
            public static string GetLocationCodeForDelivery(int docEntry)
            {
                CultureInfo dbCultureInfo = CultureInfo.InvariantCulture;
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT TOP 1 ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxSalesDeliveryNoteLineTable.Columns.StorageLocationCode.NAME));
                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteLineTable.NAME));
                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxSalesDeliveryNoteLineTable.Columns.DocEntry.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, dbCultureInfo));

                return strBuilder.ToString();
            }


        }

        #endregion


    }
}
